package com.autonavi.amapauto.gdarcameraservicedemo;

import android.content.Context;
import android.graphics.ImageFormat;
import android.hardware.camera2.CameraAccessException;
import android.hardware.camera2.CameraCaptureSession;
import android.hardware.camera2.CameraCharacteristics;
import android.hardware.camera2.CameraDevice;
import android.hardware.camera2.CameraManager;
import android.hardware.camera2.CaptureFailure;
import android.hardware.camera2.CaptureRequest;
import android.hardware.camera2.TotalCaptureResult;
import android.media.Image;
import android.media.ImageReader;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.MemoryFile;
import android.os.ParcelFileDescriptor;
import android.os.RemoteException;
import android.util.Log;
import android.util.Size;
import android.view.Surface;

import com.autonavi.amapauto.gdarcameraservice.IGDCameraStateCallBack;
import com.autonavi.amapauto.gdarcameraservice.model.ArCameraOpenResultParam;
import com.autonavi.amapauto.gdarcameraservicedemo.utils.SharedMemUtils;

import java.io.FileDescriptor;
import java.io.IOException;
import java.lang.reflect.Method;
import java.nio.ByteBuffer;
import java.util.Arrays;

public class CameraServiceHelper {

    private static final String TAG = "CameraServiceHelper";
    private static final String MEMORY_FILE = "cneeds_camera_memory";

    private final static int PREVIEW_WIDTH = 720;
    private final static int PREVIEW_HEIGHT = 1280;
    public final static int SAVE_WIDTH = 720;
    public final static int SAVE_HEIGHT = 1280;

    private boolean mIsCameraOpen = false;


    private final int MEMORY_SIZE = 3133440 + 1;

    private CameraManager mCameraManager;
    private ImageReader mImageReader;
    private CameraDevice mCameraDevice;
    private CameraCaptureSession mCameraCaptureSession;

    private String mCameraId = "0";
    private CameraCharacteristics mCameraCharacteristics;

    private int mCameraSensorOrientation = 0;        //摄像头方向
    private int mCameraFacing = CameraCharacteristics.LENS_FACING_BACK;       //默认使用后置摄像头
    private int mDisplayRotation; //手机方向

    private boolean canTakePic = true;                   //是否可以拍照
    private boolean canExchangeCamera = false;           //是否可以切换摄像头

    private Handler mCameraHandler;
    private HandlerThread handlerThread = new HandlerThread("CameraThread");

    private Size mPreviewSize = new Size(PREVIEW_WIDTH, PREVIEW_HEIGHT); //预览大小
    private Size mSavePicSize = new Size(SAVE_WIDTH, SAVE_HEIGHT);       //保存图片大小
    private IGDCameraStateCallBack mStateCallBack;

    private MemoryFile mMemoryFile;
    private Surface mSurface;

    public CameraServiceHelper() {

        try {
            mMemoryFile = new MemoryFile(MEMORY_FILE, MEMORY_SIZE);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void setStateCallBack(IGDCameraStateCallBack stateCallBack) {
        mStateCallBack =stateCallBack;
    }

    public boolean initCameraInfo(Context context, Surface surface) {
        handlerThread.start();
        mCameraHandler = new Handler(handlerThread.getLooper());
        mSurface = surface;
        mCameraManager = (CameraManager) context.getSystemService(Context.CAMERA_SERVICE);
        String[] cameraIdList = new String[0];
        try {
            cameraIdList = mCameraManager.getCameraIdList();
            if (cameraIdList.length == 0) {
                Log.d(TAG, "没有相机可用");
                return false;
            }

            for (int i = 0; i < cameraIdList.length; i++) {
                CameraCharacteristics cameraCharacteristics = mCameraManager.getCameraCharacteristics(cameraIdList[i]);
                Integer facing = cameraCharacteristics.get(CameraCharacteristics.LENS_FACING);
                if (facing == mCameraFacing) {
                    mCameraId = cameraIdList[i];
                    mCameraCharacteristics = cameraCharacteristics;
                }
                Log.d(TAG, "设备中的摄像头" + mCameraId);
            }

            Integer supportLevel = mCameraCharacteristics.get(CameraCharacteristics.INFO_SUPPORTED_HARDWARE_LEVEL);
            if (supportLevel == CameraCharacteristics.INFO_SUPPORTED_HARDWARE_LEVEL_LEGACY) {
                Log.d(TAG, "相机硬件不支持新特性");
            }

            // 获取摄像头方向
            mCameraSensorOrientation = mCameraCharacteristics.get(CameraCharacteristics.SENSOR_ORIENTATION);

            mImageReader = ImageReader.newInstance(mPreviewSize.getWidth(), mPreviewSize.getHeight(), ImageFormat.YUV_420_888, 1);
            mImageReader.setOnImageAvailableListener(onImageAvailableListener, mCameraHandler);
            return true;
        } catch (CameraAccessException e) {
            e.printStackTrace();
        }

        return false;
    }

    private ImageReader.OnImageAvailableListener onImageAvailableListener =
            new ImageReader.OnImageAvailableListener() {

                @Override
                public void onImageAvailable(ImageReader reader) {
                    // 拿到拍照照片数据
                    Image image = reader.acquireNextImage();
                    ByteBuffer byteBuffer = image.getPlanes()[0].getBuffer();
                    byte[] bytes = new byte[byteBuffer.remaining()];
                    byteBuffer.get(bytes);

                    reader.close();
                    log("拿到拍照照片数据");

                    byte[] buffer = new byte[20+bytes.length];

                    SharedMemUtils.initHeader(buffer);
                    boolean b = SharedMemUtils.canWrite(buffer);
                    if (b) {
                        SharedMemUtils.setOffset(buffer, 0);
                        SharedMemUtils.setLength(buffer, 0);
                        SharedMemUtils.setContentSize(buffer, bytes.length);
                        SharedMemUtils.setContent(buffer, bytes);
                        try {
                            // 写一次 , 读取数据后 数据会被清空
                            // 持续写,不读,数据不会清空,注意数据覆盖(offset值)
                            mMemoryFile.writeBytes(buffer, 0, 0, buffer.length);
                            Method getFileDescriptorMethod = mMemoryFile.getClass().getDeclaredMethod("getFileDescriptor");
                            if(getFileDescriptorMethod != null){
                                FileDescriptor fileDescriptor = (FileDescriptor) getFileDescriptorMethod.invoke(mMemoryFile);
                                // 序列化，才可传送
                                ParcelFileDescriptor pfd = ParcelFileDescriptor.dup(fileDescriptor);

                                ArCameraOpenResultParam openResultParam = new ArCameraOpenResultParam();
                                openResultParam.cameraId = mCameraId;
                                openResultParam.imageHeight = PREVIEW_HEIGHT;
                                openResultParam.imageWidth = PREVIEW_WIDTH;
                                mStateCallBack.onOpened(pfd, openResultParam, MEMORY_FILE);
                            }

                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                }
            };

    public void openCamera() throws CameraAccessException {

        mCameraManager.openCamera(mCameraId, new CameraDevice.StateCallback() {
            @Override
            public void onOpened(CameraDevice camera) {
                mCameraDevice = camera;
                try {
                    createCaptureSession(camera);
                } catch (CameraAccessException e) {
                    e.printStackTrace();
                }
                try {
                    mStateCallBack.onConnected();
                } catch (RemoteException e) {
                    e.printStackTrace();
                }
                mIsCameraOpen = true;
            }

            @Override
            public void onDisconnected(CameraDevice camera) {
                log("onDisconnected");
                try {
                    mStateCallBack.onDisconnected();
                } catch (RemoteException e) {
                    e.printStackTrace();
                }
                mIsCameraOpen = false;
            }

            @Override
            public void onError(CameraDevice camera, int error) {
                log("onError" +error);
                try {
                    mStateCallBack.onError(error, camera.getId());
                } catch (RemoteException e) {
                    e.printStackTrace();
                }
                mIsCameraOpen = false;
                releaseCamera();
                releaseThread();
            }
        }, mCameraHandler);
    }

    private void createCaptureSession(CameraDevice cameraDevice) throws CameraAccessException {
        final CaptureRequest.Builder builder = cameraDevice.createCaptureRequest(CameraDevice.TEMPLATE_PREVIEW);

        builder.addTarget(mImageReader.getSurface());
        builder.addTarget(mSurface);
        builder.set(CaptureRequest.CONTROL_AE_MODE, CaptureRequest.CONTROL_AE_MODE_ON_AUTO_FLASH);
        builder.set(CaptureRequest.CONTROL_AF_MODE, CaptureRequest.CONTROL_AF_MODE_CONTINUOUS_PICTURE);

        cameraDevice.createCaptureSession(Arrays.asList(mImageReader.getSurface()), new CameraCaptureSession.StateCallback() {
            @Override
            public void onConfigured(CameraCaptureSession session) {
                mCameraCaptureSession = session;
                try {
                    session.setRepeatingRequest(builder.build(), captureCallback, mCameraHandler);
                } catch (CameraAccessException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onConfigureFailed(CameraCaptureSession session) {

            }

            @Override
            public void onClosed(CameraCaptureSession session) {
                super.onClosed(session);
                releaseCamera();
                releaseThread();

                try {
                    mStateCallBack.onClosed(0, "onClosed");
                } catch (RemoteException e) {
                    e.printStackTrace();
                }
            }
        }, mCameraHandler);
    }

    public void releaseCamera() {
        if (mCameraCaptureSession != null) {
            mCameraCaptureSession.close();
            mCameraCaptureSession = null;
        }

        if (mCameraDevice != null) {
            mCameraDevice.close();
            mCameraDevice = null;
        }

        if (mImageReader != null) {
            mImageReader.close();
            mImageReader = null;
        }

        canExchangeCamera = false;
    }

    public void releaseThread() {
        handlerThread.quitSafely();
        try {
            handlerThread.join();
            handlerThread = null;
            mCameraHandler = null;
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    CameraCaptureSession.CaptureCallback captureCallback = new CameraCaptureSession.CaptureCallback() {
        @Override
        public void onCaptureCompleted(CameraCaptureSession session, CaptureRequest request, TotalCaptureResult result) {
            super.onCaptureCompleted(session, request, result);
            canExchangeCamera = true;
            canTakePic = true;
        }

        @Override
        public void onCaptureFailed(CameraCaptureSession session, CaptureRequest request, CaptureFailure failure) {
            super.onCaptureFailed(session, request, failure);
            log("onCaptureFailed");
        }
    };

    private void log(String msg) {
        Log.d(TAG, msg);
    }

    public boolean isIsCameraOpen() {
        return mIsCameraOpen;
    }

    public void setIsCameraOpen(boolean mIsCameraOpen) {
        this.mIsCameraOpen = mIsCameraOpen;
    }
}
