package com.autonavi.amapauto.gdarcameraservicedemo;

import android.app.Service;
import android.content.Intent;
import android.graphics.ImageFormat;
import android.hardware.Camera;
import android.os.IBinder;
import android.os.MemoryFile;
import android.os.ParcelFileDescriptor;
import android.os.RemoteException;
import android.util.Log;
import android.view.Surface;

import com.autonavi.amapauto.gdarcameraservice.IGDCameraService;
import com.autonavi.amapauto.gdarcameraservice.IGDCameraStateCallBack;
import com.autonavi.amapauto.gdarcameraservice.IGDSize;
import com.autonavi.amapauto.gdarcameraservice.model.ArCameraOpenResultParam;
import com.autonavi.amapauto.gdarcameraservice.model.GDArCameraParam;
import com.autonavi.amapauto.gdarcameraservicedemo.utils.SharedMemUtils;

import java.io.FileDescriptor;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.Arrays;

public class GDArCameraService extends Service {
    private final static String TAG = "GDArCameraService";
    private Camera mCamera;
    private Surface mSurface;
    private IGDCameraStateCallBack mGdCameraStateCallBack;
    private static final String MEMORY_NAME = "cneeds_camera_memory";
    public static final int HEADER_SIZE = 20;
    /**
     * 图片宽度
     * 注意： 车盒上支持1080宽度，但手机上不支持，手机上设置宽度位1080时，会报错导致相机打不开
     */
    private int imageWidth = 1280;

    /**
     * 图片高度
     */
    private int imageHeight = 720;
    /**
     * 摄像头数据大小, 视实际分辨率确定
     */
    private int dataSize = (imageWidth == 0 && imageHeight == 0) ? 1382400 :
            imageWidth * imageHeight * 3 / 2;
    private final int MEMORY_SIZE = dataSize + SharedMemUtils.HEADER_SIZE;

    private MemoryFile mMemoryFile;

    private boolean isFirstSend = true;
    private boolean mIsCameraOpened;
    private boolean mISCameraConnect;
    private Camera.Size mPreviewSize;


    @Override
    public IBinder onBind(Intent intent) {
        try {
            mMemoryFile = new MemoryFile(MEMORY_NAME, MEMORY_SIZE);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return binder;
    }

    private final IGDCameraService.Stub binder = new IGDCameraService.Stub() {
        @Override
        public boolean registerCameraStateCallback(String clientId, IGDCameraStateCallBack gdCameraStateCallBack) throws RemoteException {
            mGdCameraStateCallBack = gdCameraStateCallBack;
            return true;
        }

        @Override
        public boolean unregisterCameraStateCallback(String clientId, IGDCameraStateCallBack gdCameraStateCallBack) throws RemoteException {
            return false;
        }

        @Override
        public boolean isSupportArNavi(String clientId) throws RemoteException {
            return false;
        }

        @Override
        public IGDSize getRecommendSize(String clientId) throws RemoteException {
            return new IGDSize.Stub() {
                @Override
                public int getWidth() throws RemoteException {
                    return mPreviewSize.width;
                }

                @Override
                public int getHeight() throws RemoteException {
                    return mPreviewSize.height;
                }
            };
        }

        @Override
        public boolean isCameraConnected(String clientId) throws RemoteException {
            return mISCameraConnect;
        }

        @Override
        public boolean isCameraOpened(String clientId) throws RemoteException {
            return mIsCameraOpened;
        }

        @Override
        public boolean initCamera(String clientId, GDArCameraParam gdArCameraParam, Surface surface) throws RemoteException {
            mCamera = null;
            mSurface = surface;
            try {
                mCamera = Camera.open(); // attempt to get a Camera instance
            } catch (Exception e) {
                // Camera is not available (in use or does not exist)
                return false;
            }
            mISCameraConnect = true;
            return true;
        }

        @Override
        public boolean openCamera(String clientId) throws RemoteException {

            if (mCamera == null) {
                mIsCameraOpened = false;
                return false;
            }

            // stop preview before making changes
            try {
                mCamera.stopPreview();
            } catch (Exception e) {
                // ignore: tried to stop a non-existent preview
            }

            // set preview size and make any resize, rotate or
            // reformatting changes here

            // start preview with new settings
            try {
                if (mSurface != null) {
                    // 这里设置预览，如果不需要预览，直接注释掉就好
                    Method method = mCamera.getClass().getMethod("setPreviewSurface", Surface.class);
                    method.invoke(mCamera, mSurface);
                }

//                parameters.setPreviewSize(1080, 720); 车盒可以用这个尺寸，手机不能，手机用这个尺寸会报错
                Camera.Parameters parameters = mCamera.getParameters();
                parameters.setPreviewSize(imageWidth, imageHeight);
                parameters.setPreviewFormat(ImageFormat.NV21);
                mCamera.setParameters(parameters);
                mCamera.setPreviewCallback(new Camera.PreviewCallback() {
                    @Override
                    public void onPreviewFrame(byte[] data, Camera camera) {
                        Log.d(TAG, "onPreviewFrame: " + data.toString());
                        try {
                            sendData(data, camera);
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                });
                mCamera.startPreview();
                mPreviewSize = mCamera.getParameters().getPreviewSize();//获取尺寸,格式转换的时候要用到
                initMemoryFile();
                mIsCameraOpened = true;
            } catch (Exception e) {
                e.printStackTrace();
                Log.e(TAG, "Error starting camera preview: " + e.getMessage());
                mIsCameraOpened = false;
                return false;
            }
            return true;
        }

        @Override
        public boolean closeCamera(String clientId) throws RemoteException {
            if (mCamera != null) {
                mCamera.stopPreview();
                mCamera.setPreviewCallback(null);
            }
            mIsCameraOpened = false;
            mISCameraConnect = false;
            return true;
        }

        @Override
        public boolean unInitCamera(String clientId) throws RemoteException {
            mISCameraConnect = false;
            mIsCameraOpened = false;
            mCamera.release();
            return true;
        }

    };

    private byte[] header = new byte[SharedMemUtils.HEADER_SIZE];

    private void sendData(byte[] bytes, Camera camera) throws IOException {


        byte[] buffer = new byte[HEADER_SIZE + bytes.length];
        boolean canWrite;

        if (isFirstSend) {
            SharedMemUtils.initHeader(buffer);
            canWrite = SharedMemUtils.canWrite(buffer);
            isFirstSend = false;
        } else {
            mMemoryFile.readBytes(header, 0, 0, SharedMemUtils.HEADER_SIZE);
            canWrite = SharedMemUtils.canWrite(header);
        }

        /** 判断共享内存是否可写 */
        if (canWrite) {
            SharedMemUtils.setOffset(buffer, 0);
            SharedMemUtils.setLength(buffer, 0);
            SharedMemUtils.setContentSize(buffer, bytes.length);
            SharedMemUtils.setContent(buffer, bytes);
            try {
                SharedMemUtils.setCanRead(buffer);
                mMemoryFile.writeBytes(buffer, 0, 0, buffer.length);
                Log.d(TAG, "CNEEDS_TEST sendData: " + Arrays.toString(buffer));
            } catch (Exception e) {
                e.printStackTrace();
                Log.d(TAG, "sendDataError: " + e.getMessage());

            }
        }
    }

    private void initMemoryFile() throws Exception {
        Method getFileDescriptorMethod = mMemoryFile.getClass().getDeclaredMethod("getFileDescriptor");
        if (getFileDescriptorMethod != null) {
            FileDescriptor fileDescriptor = (FileDescriptor) getFileDescriptorMethod.invoke(mMemoryFile);
            // 序列化，才可传送
            ParcelFileDescriptor pfd = ParcelFileDescriptor.dup(fileDescriptor);

            ArCameraOpenResultParam openResultParam = new ArCameraOpenResultParam();
            openResultParam.cameraId = "" + getCameraId();
            openResultParam.imageHeight = mPreviewSize.height;
            openResultParam.imageWidth = mPreviewSize.width;
            /**
             * 参考 {@link com.autonavi.amapauto.gdarcameraservice.ImageFormat}
             * NV21在高德协议对应 0
             */
            openResultParam.imageFormat = 0;
            mGdCameraStateCallBack.onOpened(pfd, openResultParam, MEMORY_NAME);
        }
    }

    private int getCameraId() {
        int cameraId = 0;
        int numberOfCameras = Camera.getNumberOfCameras();
        for (int i = 0; i <= numberOfCameras; i++) {
            Camera.CameraInfo info = new Camera.CameraInfo();
            Camera.getCameraInfo(i, info);
            if (info.facing == Camera.CameraInfo.CAMERA_FACING_FRONT) {
                cameraId = i;
                break;
            }
        }
        return cameraId;
    }
}
