package com.example.cameraservicedemo;

import android.app.Activity;
import android.content.Context;
import android.content.res.Configuration;
import android.graphics.ImageFormat;
import android.graphics.SurfaceTexture;
import android.hardware.camera2.CameraAccessException;
import android.hardware.camera2.CameraCaptureSession;
import android.hardware.camera2.CameraCharacteristics;
import android.hardware.camera2.CameraDevice;
import android.hardware.camera2.CameraManager;
import android.hardware.camera2.CaptureFailure;
import android.hardware.camera2.CaptureRequest;
import android.hardware.camera2.TotalCaptureResult;
import android.hardware.camera2.params.StreamConfigurationMap;
import android.media.Image;
import android.media.ImageReader;
import android.os.Handler;
import android.os.HandlerThread;
import android.util.Log;
import android.util.Size;
import android.view.Surface;
import android.view.TextureView;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class Camera2Helper {

    private static final String TAG = "Camera2Helper";
    private final Activity mActivity;
    private final TextureView mTextureView;

    private final static int PREVIEW_WIDTH = 720;
    private final static int PREVIEW_HEIGHT = 1280;
    private final static int SAVE_WIDTH = 720;
    private final static int SAVE_HEIGHT = 1280;

    private CameraManager mCameraManager;
    private ImageReader mImageReader;
    private CameraDevice mCameraDevice;
    private CameraCaptureSession mCameraCaptureSession;

    private String mCameraId = "0";
    private CameraCharacteristics mCameraCharacteristics;

    private int mCameraSensorOrientation = 0;        //摄像头方向
    private int mCameraFacing = CameraCharacteristics.LENS_FACING_BACK;       //默认使用后置摄像头
    private int mDisplayRotation; //手机方向

    private boolean canTakePic = true;                   //是否可以拍照
    private boolean canExchangeCamera = false;           //是否可以切换摄像头

    private Handler mCameraHandler;
    private HandlerThread handlerThread = new HandlerThread("CameraThread");

    private Size mPreviewSize = new Size(PREVIEW_WIDTH, PREVIEW_HEIGHT); //预览大小
    private Size mSavePicSize = new Size(SAVE_WIDTH, SAVE_HEIGHT);       //保存图片大小


    public Camera2Helper(Activity activity, TextureView textureView) {
        mActivity = activity;
        mTextureView = textureView;
        mDisplayRotation = mActivity.getWindowManager().getDefaultDisplay().getRotation();
        handlerThread.start();
        mCameraHandler = new Handler(handlerThread.getLooper());

        mTextureView.setSurfaceTextureListener(new TextureView.SurfaceTextureListener() {
            @Override
            public void onSurfaceTextureAvailable(SurfaceTexture surface, int width, int height) {
                initCameraInfo();
            }

            @Override
            public void onSurfaceTextureSizeChanged(SurfaceTexture surface, int width, int height) {

            }

            @Override
            public boolean onSurfaceTextureDestroyed(SurfaceTexture surface) {
                releaseCamera();
                return true;
            }

            @Override
            public void onSurfaceTextureUpdated(SurfaceTexture surface) {

            }
        });
    }

    public void initCameraInfo() {
        mCameraManager = (CameraManager) mActivity.getSystemService(Context.CAMERA_SERVICE);
        String[] cameraIdList = new String[0];
        try {
            cameraIdList = mCameraManager.getCameraIdList();
            if (cameraIdList.length == 0) {
                Log.d(TAG, "没有相机可用");
                return;
            }

            for (int i = 0; i < cameraIdList.length; i++) {
                CameraCharacteristics cameraCharacteristics = mCameraManager.getCameraCharacteristics(cameraIdList[i]);
                Integer facing = cameraCharacteristics.get(CameraCharacteristics.LENS_FACING);
                if (facing == mCameraFacing) {
                    mCameraId = cameraIdList[i];
                    mCameraCharacteristics = cameraCharacteristics;
                }
                Log.d(TAG, "设备中的摄像头" + mCameraId);
            }

            Integer supportLevel = mCameraCharacteristics.get(CameraCharacteristics.INFO_SUPPORTED_HARDWARE_LEVEL);
            if (supportLevel == CameraCharacteristics.INFO_SUPPORTED_HARDWARE_LEVEL_LEGACY) {
                Log.d(TAG, "相机硬件不支持新特性");
            }

            // 获取摄像头方向
            mCameraSensorOrientation = mCameraCharacteristics.get(CameraCharacteristics.SENSOR_ORIENTATION);
            //获取StreamConfigurationMap，它是管理摄像头支持的所有输出格式和尺寸
            StreamConfigurationMap configurationMap = mCameraCharacteristics.get(CameraCharacteristics.SCALER_STREAM_CONFIGURATION_MAP);
            Size[] savePicSize = configurationMap.getOutputSizes(ImageFormat.JPEG);
            Size[] previewSize = configurationMap.getOutputSizes(SurfaceTexture.class);

            boolean exchange = exchangeWidthAndHeight(mDisplayRotation, mCameraSensorOrientation);

            mSavePicSize = getBestSize(exchange ? mSavePicSize.getHeight() : mSavePicSize.getWidth(),
                    exchange ? mSavePicSize.getWidth() : mSavePicSize.getHeight(),
                    exchange ? mTextureView.getHeight() : mTextureView.getWidth(),
                    exchange ? mTextureView.getWidth() : mTextureView.getHeight(),
                    Arrays.asList(savePicSize));

            mPreviewSize = getBestSize(exchange ? mPreviewSize.getHeight() : mPreviewSize.getWidth(),
                    exchange ? mPreviewSize.getWidth() : mPreviewSize.getHeight(),
                    exchange ? mTextureView.getHeight() : mTextureView.getWidth(),
                    exchange ? mTextureView.getWidth() : mTextureView.getHeight(),
                    Arrays.asList(previewSize));
            mTextureView.getSurfaceTexture().setDefaultBufferSize(mPreviewSize.getWidth(), mPreviewSize.getHeight());
            log("预览最优尺寸 ：" + mPreviewSize.getWidth() * mPreviewSize.getHeight() + ", 比例  " + (float) mPreviewSize.getWidth() / mPreviewSize.getHeight());
            log("保存图片最优尺寸 ：" + mSavePicSize.getWidth() * mSavePicSize.getHeight() + ", 比例  " + (float) mSavePicSize.getWidth() / mSavePicSize.getHeight());

            //根据预览的尺寸大小调整TextureView的大小，保证画面不被拉伸
//            int orientation = mActivity.getResources().getConfiguration().orientation;
//            if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
//                mTextureView.setRotationX(mPreviewSize.getWidth());
//                mTextureView.setRotationY(mPreviewSize.getHeight());
//            } else {
//                mTextureView.setRotationX(mPreviewSize.getHeight());
//                mTextureView.setRotationX(mPreviewSize.getWidth());
//            }

            mImageReader = ImageReader.newInstance(mPreviewSize.getWidth(), mPreviewSize.getHeight(), ImageFormat.JPEG, 1);
            mImageReader.setOnImageAvailableListener(onImageAvailableListener, mCameraHandler);

            openCamera();
        } catch (CameraAccessException e) {
            e.printStackTrace();
        }
    }

    private ImageReader.OnImageAvailableListener onImageAvailableListener =
            new ImageReader.OnImageAvailableListener() {

                @Override
                public void onImageAvailable(ImageReader reader) {
                    // 拿到拍照照片数据
                    Image image = reader.acquireNextImage();
                    ByteBuffer byteBuffer = image.getPlanes()[0].getBuffer();
                    byte[] bytes = new byte[byteBuffer.remaining()];
                    byteBuffer.get(bytes);
                    reader.close();
                    log("拿到拍照照片数据");
                }
            };

    private void openCamera() throws CameraAccessException {

        mCameraManager.openCamera(mCameraId, new CameraDevice.StateCallback() {
            @Override
            public void onOpened(CameraDevice camera) {
                mCameraDevice = camera;
                try {
                    createCaptureSession(camera);
                } catch (CameraAccessException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onDisconnected(CameraDevice camera) {
                log("onDisconnected");
            }

            @Override
            public void onError(CameraDevice camera, int error) {
                log("onError" +error);
            }
        }, mCameraHandler);
    }

    private void createCaptureSession(CameraDevice cameraDevice) throws CameraAccessException {
        final CaptureRequest.Builder builder = cameraDevice.createCaptureRequest(CameraDevice.TEMPLATE_PREVIEW);

        Surface surface = new Surface(mTextureView.getSurfaceTexture());
        builder.addTarget(surface);
//        builder.addTarget(mImageReader.getSurface());
        builder.set(CaptureRequest.CONTROL_AE_MODE, CaptureRequest.CONTROL_AE_MODE_ON_AUTO_FLASH);
        builder.set(CaptureRequest.CONTROL_AF_MODE, CaptureRequest.CONTROL_AF_MODE_CONTINUOUS_PICTURE);

        cameraDevice.createCaptureSession(Arrays.asList(surface, mImageReader.getSurface()), new CameraCaptureSession.StateCallback() {
            @Override
            public void onConfigured(CameraCaptureSession session) {
                mCameraCaptureSession = session;
                try {
                    session.setRepeatingRequest(builder.build(), captureCallback, mCameraHandler);
                } catch (CameraAccessException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onConfigureFailed(CameraCaptureSession session) {

            }

            @Override
            public void onClosed(CameraCaptureSession session) {
                super.onClosed(session);
                releaseCamera();
                releaseThread();
            }
        }, mCameraHandler);
    }

    public void releaseCamera() {
        if (mCameraCaptureSession != null) {
            mCameraCaptureSession.close();
            mCameraCaptureSession = null;
        }

        if (mCameraDevice != null) {
            mCameraDevice.close();
            mCameraDevice = null;
        }

        if (mImageReader != null) {
            mImageReader.close();
            mImageReader = null;
        }

        canExchangeCamera = false;
    }

    public void releaseThread() {
        if (handlerThread == null) {
            return;
        }
        handlerThread.quitSafely();
        try {
            handlerThread.join();
            handlerThread = null;
            mCameraHandler = null;
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    CameraCaptureSession.CaptureCallback captureCallback = new CameraCaptureSession.CaptureCallback() {
        @Override
        public void onCaptureCompleted(CameraCaptureSession session, CaptureRequest request, TotalCaptureResult result) {
            super.onCaptureCompleted(session, request, result);
            canExchangeCamera = true;
            canTakePic = true;
        }

        @Override
        public void onCaptureFailed(CameraCaptureSession session, CaptureRequest request, CaptureFailure failure) {
            super.onCaptureFailed(session, request, failure);
            log("onCaptureFailed");
        }
    };

    /**
     * 根据提供的屏幕方向 [displayRotation] 和相机方向 [sensorOrientation] 返回是否需要交换宽高
     */
    private boolean exchangeWidthAndHeight(int displayRotation, int sensorOrientation) {
        boolean exchange = false;
        switch (displayRotation) {
            case Surface.ROTATION_0:
            case Surface.ROTATION_180:
                if (sensorOrientation == 90 || sensorOrientation == 270) {
                    exchange = true;
                }
                break;
            case Surface.ROTATION_90:
            case Surface.ROTATION_270:
                if (sensorOrientation == 0 || sensorOrientation == 180) {
                    exchange = true;
                }
                break;
            default:
                log("Display rotation is invalid: " + displayRotation);
                break;
        }
        log("屏幕方向" + displayRotation);
        log("相机方向" + sensorOrientation);
        return exchange;
    }

    private void log(String msg) {
        Log.d(TAG, msg);
    }

    /**
     * 根据提供的参数值返回与指定宽高相等或最接近的尺寸
     *
     * @param targetWidth  目标宽度
     * @param targetHeight 目标高度
     * @param maxWidth     最大宽度(即TextureView的宽度)
     * @param maxHeight    最大高度(即TextureView的高度)
     * @param sizeList     支持的Size列表
     * @return 返回与指定宽高相等或最接近的尺寸
     */
    private Size getBestSize(int targetWidth, int targetHeight, int maxWidth, int maxHeight, List<Size> sizeList) {
        ArrayList bigEnough = new ArrayList<Size>();     //比指定宽高大的Size列表
        ArrayList notBigEnough = new ArrayList<Size>();  //比指定宽高小的Size列表

        for (int i = 0; i < sizeList.size(); i++) {
            Size size = sizeList.get(i);

            //宽<=最大宽度  &&  高<=最大高度  &&  宽高比 == 目标值宽高比
            if (size.getWidth() <= maxWidth && size.getHeight() <= maxHeight
                    && size.getWidth() == size.getHeight() * targetWidth / targetHeight) {

                if (size.getWidth() >= targetWidth && size.getHeight() >= targetHeight)
                    bigEnough.add(size);
                else
                    notBigEnough.add(size);
            }
            log("系统支持的尺寸: " + size.getWidth() * size.getHeight() + " ,  比例 ：" + (float) size.getWidth() / size.getHeight());
        }

        log("最大尺寸 ：" + maxWidth * maxHeight + ", 比例 ：" + (float) targetWidth / targetHeight);
        log("目标尺寸 ：" + targetWidth * targetHeight + ", 比例 ：" + (float) targetWidth / targetHeight);

        //选择bigEnough中最小的值  或 notBigEnough中最大的值
        Size size;
        if (bigEnough.size() > 0) {
            size = (Size) Collections.min(bigEnough, new CompareSizesByArea());
        } else if (notBigEnough.size() > 0) {
            size = (Size) Collections.min(notBigEnough, new CompareSizesByArea());
        } else {
            size = sizeList.get(0);
        }

        return size;
    }


    private class CompareSizesByArea implements Comparator<Size> {

        @Override
        public int compare(Size size1, Size size2) {
            return Long.signum((long) size1.getWidth() * size1.getHeight() - (long) size2.getWidth() * size2.getHeight());
        }
    }


}
